@@ -2,6 +2,12 @@
 from __future__ import print_function
 
 import threading
+# Python3 queue support.
+try:
+    import Queue
+except ImportError:
+    import queue as Queue
+import time
 
 import re
 from telebot import apihelper, types
@@ -13,6 +19,47 @@ Module : telebot
 API_URL = r"https://api.telegram.org/"
 
 
+class ThreadPool:
+    class WorkerThread(threading.Thread):
+        count = 0
+
+        def __init__(self, queue):
+            threading.Thread.__init__(self, name="WorkerThread{0}".format(self.__class__.count + 1))
+            self.__class__.count += 1
+            self.queue = queue
+            self.daemon = True
+
+            self._running = True
+            self.start()
+
+        def run(self):
+            while self._running:
+                try:
+                    task, args, kwargs = self.queue.get(False)
+                    task(*args, **kwargs)
+                except Queue.Empty:
+                    time.sleep(0)
+                    pass
+
+        def stop(self):
+            self._running = False
+
+    def __init__(self, num_threads=4):
+        self.tasks = Queue.Queue()
+        self.workers = [self.WorkerThread(self.tasks) for _ in range(num_threads)]
+
+        self.num_threads = num_threads
+
+    def put(self, func, *args, **kwargs):
+        self.tasks.put((func, args, kwargs))
+
+    def close(self):
+        for worker in self.workers:
+            worker.stop()
+        for worker in self.workers:
+            worker.join()
+
+
 class TeleBot:
     """ This is TeleBot Class
     Methods:
@@ -30,16 +77,25 @@ class TeleBot:
         getUpdates
     """
 
-    def __init__(self, token):
+    def __init__(self, token, create_threads=True, num_threads=4):
+        """
+
+        :param token: bot API token
+        :param create_threads: Create thread for message handler
+        :param num_threads: Number of worker in thread pool.
+        :return:
+        """
         self.token = token
         self.update_listener = []
         self.polling_thread = None
         self.__stop_polling = False
-        self.interval = 3
-
         self.last_update_id = 0
+        self.num_threads = num_threads
+        self.__create_threads = create_threads
 
         self.message_handlers = []
+        if self.__create_threads:
+            self.worker_pool = ThreadPool(num_threads)
 
     def get_update(self):
         """
@@ -61,8 +117,10 @@ class TeleBot:
 
     def __notify_update(self, new_messages):
         for listener in self.update_listener:
-            t = threading.Thread(target=listener, args=new_messages)
-            t.start()
+            if self.__create_threads:
+                self.worker_pool.put(listener, new_messages)
+            else:
+                listener(new_messages)
 
     def polling(self):
         """
@@ -287,13 +345,14 @@ class TeleBot:
 
         Examples:
         extract_command('/help'): 'help'
+        extract_command('/help@BotName'): 'help'
         extract_command('/search black eyed peas'): 'search'
         extract_command('Good day to you'): None
 
         :param text: String to extract the command from
         :return: the command if `text` is a command, else None.
         """
-        return text.split()[0][1:] if TeleBot.is_command(text) else None
+        return text.split()[0].split('@')[0][1:] if TeleBot.is_command(text) else None
 
     @staticmethod
     def _test_message_handler(message_handler, message):
@@ -312,8 +371,12 @@ class TeleBot:
         for message in new_messages:
             for message_handler in self.message_handlers:
                 if self._test_message_handler(message_handler, message):
-                    t = threading.Thread(target=message_handler['function'], args=(message,))
-                    t.start()
+                    if self.__create_threads:
+                        self.worker_pool.put(message_handler['function'], message)
+                        # t = threading.Thread(target=message_handler['function'], args=(message,))
+                        # t.start()
+                    else:
+                        message_handler['function'](message)
                     break
 
 
